--[[ ==== Protocol handler =============================================================================== ]]--

FlagRSPProtocol = {
    timeForOnline = 300; -- 5 min
    FILTER_TAGS = true;
};

local isAlive = false;

function FlagRSPProtocol:IsAlive()
    return isAlive;
end

--[[ Actions on player logout event

]]--
function FlagRSPProtocol:OnLogout()
    if not self:IsAlive() then
        return;
    end
    
    self:Save( FlagRSPData )
end

function FlagRSPProtocol:Save( saveToVar )
    if not self:IsAlive() then
        return;
    end


    -- Saving msp data
    saveToVar.msp = {};
    saveToVar.msp.my = {};
    saveToVar.msp.my = msp.my;
    saveToVar.msp.myver = {};
    saveToVar.msp.myver = msp.myver;

end

function FlagRSPProtocol:Load( loadToVar )
    
    FlagRSPProtocol:LoadMsp(loadToVar);
end

function FlagRSPProtocol:IsValidProtocolTarget(unitID)
    if ( UnitExists(unitID) and UnitIsPlayer(unitID) and not (UnitIsUnit(unitID,"player") == 1)  and UnitFactionGroup("player") == UnitFactionGroup(unitID) and UnitIsConnected(unitID) ) then
        return true;
    elseif ( (strfind(unitID,"^party") or strfind(unitID, "^raid") or strfind(unitID, "^focus")) and UnitIsPlayer(unitID) and UnitIsConnected(unitID) ) then
        return true;
    else
        return false;
    end
end

--[[ Request to relative unitID
    - unit = "mouseover", "target"
    - fields = MSP based field names, one string ("DE") or in a table { "DE", "NA" }
               alternatively use "ott" (Tooltip), "oib" (InfoBox), "ofb" (FriendInfoBox) for predefined tables
    
    Returns: boolean, whether the request was successfull or not
]]--
function FlagRSPProtocol:Request( unit, fields )
    if not self:IsAlive()  then
        return false;
    end

    local request = false;
    local ok = false;

    request = self:IsValidProtocolTarget(unit);
    -- if ( UnitExists(unit) and UnitIsPlayer(unit) and not (UnitIsUnit(unit,"player") == 1)  and UnitFactionGroup("player") == UnitFactionGroup(unit) and UnitIsConnected(unit) ) then
        -- ok = true;
    -- elseif ( (strfind(unit,"^party") or strfind(unit, "^raid") or strfind(unit, "^focus")) and UnitIsPlayer(unit) and UnitIsConnected(unit) ) then
        -- ok = true;
    -- else
        -- return false;
    -- end
    
    if request then
        local unitName, unitRealm = UnitName(unit);
        
        local reqName = "";
        
        if ( unitRealm == nil or unitRealm == "" )then
            reqName = unitName;
        elseif ( unitRealm ~= nil and unitRealm ~= "" ) then
            reqName = unitName .. "-" .. unitRealm;
        end

        local fieldTable;
        
        if ( fields == "ott" ) then -- Tooltip data
            fieldTable = self.mspSupportedTTFields;
        elseif ( fields == "oib" ) then -- Info box data
            fieldTable = self.mspSupportedIBFields;
        elseif ( fields == "ofb" ) then -- Friend box data
            fieldTable = self.mspSupportedIBFields;
        else
            fieldTable = fields or self.mspSupportedTTFields;
        end
        
        request = msp:Request(reqName, fieldTable);
    end

    return request;
end


--[[ Request direct to playerID
    - name = name of a player, realm foreigners are noted as "player-realm"
    - fields = table with fields to request
    
    Returns: boolean, whether the request was successfull or not
]]--
function FlagRSPProtocol:RequestDirect( name , fields )
    if not self:IsAlive() then
        return false;
    end
    
    local request = false;

    request = msp:Request(name, fields);
    
    return request;
end


--[[ Update Flag, Tooltip with new infos
]]--
function FlagRSPProtocol:Update( unitType, unit, request )
    if not self:IsAlive() then
        return;
    end
    
    if ( request ) then
        local unitName, unitRealm = UnitName(unit);
        local reqName = "";
        if ( unitType == "callback" ) then
            reqName = unit;
        elseif ( unitType == "isonline" ) then
            reqName = unit;
        elseif ( unitRealm ~= nil and unitRealm ~= "" ) then
            reqName = tostring(unitName) .. "-" .. tostring(unitRealm);
        else
            reqName = unitName;
        end
        
        -- Filter ui escapes sequences out from incoming flags
        if self.FILTER_TAGS then
            for k,v in pairs(msp.char[reqName].field) do
                -- print("["..k.."] = "..v)
                msp.char[reqName].field[k] = strtrim(FlagRSPProtocol:FilterIncoming( msp.char[reqName].field[k] ))
            end
        end
        
        FlagRSPProtocol:UpdateFlagList2( reqName, msp.char[reqName].field, msp.char[reqName].ver, "libmsp", unitRealm);
        FlagRSP:UpdateTooltip(true);
    end
end


-- UpdateOwn
function FlagRSPProtocol:UpdateOwn()
    if not self:IsAlive() then
        return;
    end
    
    if ( isAlive ) then 
        -- Update MSP Data
        msp.my["NA"] = FlagRSPConfigure:GetFullname();
        msp.my["NT"] = FlagRSPConfigure:GetTitle();
        msp.my["DE"] = FlagRSPConfigure:GetDescription();
        msp.my["FR"] = tostring(FlagRSPConfigure:GetRPFlag());
        msp.my["FC"] = tostring(FlagRSPConfigure:GetCharacterStatus());
        msp:Update();

        -- Update FlagList2
        local pName, pRealm = UnitName("player");
        
        FlagRSPProtocol:UpdateFlagList2( pName, msp.my, msp.myver, "libmspintern", pRealm);
    -- else
        -- local pName, pRealm = UnitName("player");
        -- local fields = {};
        -- fields["NA"] = FlagRSPConfigure:GetFullname();
        -- fields["NT"] = FlagRSPConfigure:GetTitle();
        -- fields["DE"] = FlagRSPConfigure:GetDescription();
        -- fields["FR"] = tostring(FlagRSPConfigure:GetRPFlag());
        -- fields["FC"] = tostring(FlagRSPConfigure:GetCharacterStatus());
        
        -- local rev = {};
        -- rev["NA"] = 1;
        -- rev["NT"] = 1;
        -- rev["DE"] = 1;
        -- rev["FR"] = 1;
        -- rev["FC"] = 1;
        
        -- FlagRSPProtocol:UpdateFlagList2( pName, fields, rev, "libmspintern", pRealm);
    end
end


function FlagRSPProtocol:IsSupported( name )
    if not self:IsAlive() then
        return false;
    end
    
    if ( msp.char[name] == nil ) then
        return false;
    end
    
    return msp.char[name].supported;
end


--[[ Checks if "ingameName" is online, uses protocol for check
    Returns
        - 1 if online
        - -1 if offline
        - 0 if offline and no information on this guy (maybe protocol not supported)
]]--
function FlagRSPProtocol:CheckIsOnline( ingameName, realm )
    if not self:IsAlive() then
        return 0;
    end
    
    if ingameName == nil then return; end
    
    local isOnline = 0;
    realm = FlagHandler:GetRealm(realm);

    -- if ( ingameName ~= nil and FlagList2[realm][ingameName] ~= nil ) then
    if ( ingameName ~= nil and FlagHandler:IsSetEntry( ingameName, realm ) ) then
        local lastFlagTime = FlagHandler:GetFlag("timeStamp", ingameName, nil);
        local now = time();

        if ( lastFlagTime ~= nil ) then -- Entry exists
            local lastSeen = now - tonumber(lastFlagTime);
            if ( lastSeen <= self.timeForOnline ) then
                isOnline = 1;
            else
                isOnline = -1;
            end
        end
        if ( isOnline ~= 1 ) then -- Else ask for entry
            local request = FlagRSPProtocol:RequestDirect( ingameName , self.mspSupportedTTFields );
            FlagRSPProtocol:Update("isonline", ingameName, request)
        end
    end

    return isOnline;
end


function FlagRSPProtocol:UpdateFlagList2( ingameName, fields, revisions, protocol, realm )
    if ( protocol == nil or protocol == "libmsp" or protocol == "libmspintern") then
        local c, r = fields, revisions;
        if (protocol == "libmsp") then
            -- Check if data supported
            if ( not FlagRSPProtocol:IsSupported( ingameName ) ) then
                return;
            end
        end

        -- Full name
        if ( fields["NA"] ~= nil ) then
            FlagHandler:AddFlag("Fullname", fields["NA"], ingameName, revisions["NA"], realm);
        end

        -- Title
        if ( fields["NT"] ~= nil ) then
            FlagHandler:AddFlag("Title", fields["NT"], ingameName, revisions["NT"], realm);
        end

        -- Flag RP
        if ( fields["FR"] ~= nil ) then
            FlagHandler:AddFlag("RPFlag", tonumber(fields["FR"]), ingameName, revisions["FR"], realm);
        end

        -- Flag Character
        if ( fields["FC"] ~= nil ) then
            FlagHandler:AddFlag("CharFlag", fields["FC"], ingameName, revisions["FC"], realm);
        end

        -- Used Flagaddon
        if ( fields["VA"] ~= nil ) then
            FlagHandler:AddFlag("FlagAddon", fields["VA"], ingameName, revisions["VA"], realm);
            -- Check FlagRSP Version
            local name, ver,_ = strsplit("/",fields["VA"],3);
            if ( name == "flagRSP" ) then
                FlagRSP:checkVersion(ver);
            end
        end
        
        -- Currently
        if ( fields["CU"] ~= nil ) then
            FlagHandler:AddFlag("Current", fields["CU"], ingameName, revisions["CU"], realm);
        end        

        -- Desciption
        if ( fields["DE"] ~= nil ) then
            FlagHandler:AddFlag("Desc", fields["DE"], ingameName, revisions["DE"], realm);
        end

        -- Versions are updated in FlagHandler:addFlag
    end
end

function FlagRSPProtocol:FilterIncoming( text )
    local s = "";
    s = text;

    -- Remove color codes
    s = gsub(s,"|c%x%x%x%x%x%x%x%x","")
    s = gsub(s,"|r","")

    -- Remove textures
    s =  gsub(s,"|T.-|t","")

    -- Remove links
    s =  gsub(s,"|H.-|h","")
    s = gsub(s, "|h", "");

    return s;
end


--[[ ==== Library support for LibMSP ===================================================================== ]]--
FlagRSPProtocol.mspSupportedTTFields = { "NA", "NT", "FR", "FC", "VA", "DE", "CU" }; -- Tooltip
FlagRSPProtocol.mspSupportedIBFields = { "NA", "NT", "FR", "FC", "VA", "DE", "CU" }; -- InfoBox


function FlagRSPProtocol.CallbackMSP( ingameID )
    if ( not FlagRSPProtocol:IsAlive() ) then
        return;
    end
    
    FlagRSPProtocol:Update("callback", ingameID, true);
end


function FlagRSPProtocol:LoadMsp(loadToVar)
    if ( not self:IsAlive() ) then
        return;
    end
    
    local version = FlagRSP.verMajor .. "." .. FlagRSP.verMinor .. "." .. FlagRSP.verPatch;
    
    msp.my.VA = "flagRSP/"..version or "";
    
    msp.my.GU = UnitGUID("player");
    msp.my.GS = tostring( UnitSex("player"));
    msp.my.GC = select( 2, UnitClass("player"));
    msp.my.GR = select( 2, UnitRace("player"));
    
    -- FlagRSPData must be available!
    if (loadToVar ~= nil ) then
        msp.my.NA = loadToVar.Fullname or "";
        msp.my.NT = loadToVar.Title or "";
        msp.my.DE = loadToVar.CharDesc or "";
        msp.my.FR = loadToVar.RPFlag or "0";
        msp.my.FC = loadToVar.CharStatus or "0";
        
        if(loadToVar.msp ~= nil ) then
            if ( loadToVar.msp.myver ~= nil ) then
                msp.myver = loadToVar.msp.myver;
            end
        end
    end
    
    msp:Update();
end

-- Detect any other MSP AddOn and bail out in case of conflict, based on Etarna, http://moonshyne.org/msp/
-- _G.msp_RPAddOn = "boesesImWegAddon";

local FlagRSP_Locale_MSP_CONFLICT = "You can only use one MSP AddOn at once, but you have both flagRSP and %s loaded.\n\nAll MSP AddOns can communicate with each other, but please do not try to use more than one at once as conflicts might arise.";
local FlagRSP_Locale_MSP_CONFLICTADDON = "another MSP AddOn";

if _G.msp_RPAddOn or _G.mrp then
	StaticPopupDialogs[ "FLAGRSP_MSP_CONFLICT" ] = {
        preferredIndex = 3,
		text = format( "|cffff0000" ..ERROR_CAPS.."|r: " .. FlagRSP_Locale_MSP_CONFLICT, tostring(_G.msp_RPAddOn) or FlagRSP_Locale_MSP_CONFLICTADDON ),
		button1 = OKAY or "OK",
		whileDead = true,
		timeout = 0,
	};
	StaticPopup_Show( "FLAGRSP_MSP_CONFLICT" );
    
    isAlive = false;
else
    _G.msp_RPAddOn = "flagRSP MoP";
    isAlive = true;
    
    table.insert( msp.callback.received, FlagRSPProtocol.CallbackMSP );
end 